﻿#nullable disable

using AdoNetWrapper.HardCodedSample.Common;
using AdoNetWrapperSamples.HardCodedSample.EntityClasses;
using System.Data;
using System.Data.SqlClient;

namespace AdoNetWrapperSamples.HardCodedSample.RepositoryClasses;

public class ProductRepository {
  public virtual List<Product> Search(string connectString, string sql) {
    List<Product> ret;

    // Create a connection
    using SqlConnection cnn = new(connectString);

    // Create a command object
    using SqlCommand cmd = new(sql, cnn);
    // Open the connection
    cnn.Open();
    // Create a data reader within a using
    // so it is closed and disposed of properly
    using SqlDataReader rdr = cmd.ExecuteReader(CommandBehavior.CloseConnection);

    // Build the collection of entity objects
    ret = BuildEntityList(rdr);

    return ret;
  }

  protected virtual List<Product> BuildEntityList(IDataReader rdr) {
    List<Product> ret = new();

    // Loop through all rows in the data reader
    while (rdr.Read()) {
      // Create new object and add to collection
      ret.Add(new Product {
        ProductID = rdr.GetData<int>("ProductID"),
        Name = rdr.GetData<string>("Name"),
        ProductNumber = rdr.GetData<string>("ProductNumber"),
        Color = rdr.GetData<string>("Color"),
        StandardCost = rdr.GetData<decimal>("StandardCost"),
        ListPrice = rdr.GetData<decimal>("ListPrice"),
        SellStartDate = rdr.GetData<DateTime>("SellStartDate"),
        SellEndDate = rdr.GetData<DateTime?>("SellEndDate"),
        DiscontinuedDate = rdr.GetData<DateTime?>("DiscontinuedDate")
      });
    }

    return ret;
  }
}
